@@ -134,9 +134,9 @@ const GenericIRCConversation = {
     return this._account.maxMessageLength -
            this._account.countBytes(baseMessage);
   },
-  sendMsg: function(aMessage) {
+  prepareForSending: function(aOutgoingMessage, aCount) {
     // Split the message by line breaks and send each one individually.
-    let messages = aMessage.split(/[\r\n]+/);
+    let messages = aOutgoingMessage.message.split(/[\r\n]+/);
 
     let maxLength = this.getMaxMessageLength();
 
@@ -159,24 +159,27 @@ const GenericIRCConversation = {
                       message.substr((index + 1) || maxLength));
     }
 
-    // Send each message and display it in the conversation.
-    for (let message of messages) {
-      if (!message.length)
-        return;
+    if (aCount)
+      aCount.value = messages.length;
 
-      if (!this._account.sendMessage("PRIVMSG", [this.name, message])) {
-        this.writeMessage(this._account._currentServerName,
-                          _("error.sendMessageFailed"),
-                          {error: true, system: true});
-        break;
-      }
+    return messages;
+  },
+  sendMsg: function(aMessage) {
+    if (!aMessage.length)
+      return;
 
-      // Since the server doesn't send us a message back, just assume the
-      // message was received and immediately show it.
-      this.writeMessage(this._account._nickname, message, {outgoing: true});
-
-      this._pendingMessage = true;
+    if (!this._account.sendMessage("PRIVMSG", [this.name, aMessage])) {
+      this.writeMessage(this._account._currentServerName,
+                        _("error.sendMessageFailed"),
+                        {error: true, system: true});
+      return;
     }
+
+    // Since the server doesn't send us a message back, just assume the
+    // message was received and immediately show it.
+    this.writeMessage(this._account._nickname, aMessage, {outgoing: true});
+
+    this._pendingMessage = true;
   },
   // IRC doesn't support typing notifications, but it does have a maximum
   // message length.
@@ -288,11 +291,6 @@ ircChannel.prototype = {
                                                aProperties);
   },
 
-  // Stores the prplIChatRoomFieldValues required to join this channel
-  // to enable later reconnections. If absent, the MUC will not be reconnected
-  // automatically after disconnections.
-  _chatRoomFields: null,
-
   // Section 3.2.2 of RFC 2812.
   part: function(aMessage) {
     let params = [this.name];
@@ -306,7 +304,7 @@ ircChannel.prototype = {
     this._account.sendMessage("PART", params);
 
     // Remove reconnection information.
-    delete this._chatRoomFields;
+    delete this.chatRoomFields;
   },
 
   close: function() {
@@ -446,8 +444,8 @@ ircChannel.prototype = {
           let key = getNextParam();
           // A new channel key was set, display a message if this key is not
           // already known.
-          if (this._chatRoomFields &&
-              this._chatRoomFields.getValue("password") == key) {
+          if (this.chatRoomFields &&
+              this.chatRoomFields.getValue("password") == key) {
             continue;
           }
           msg = _("message.channelKeyAdded", aSetter, key);
@@ -458,7 +456,7 @@ ircChannel.prototype = {
 
         this.writeMessage(aSetter, msg, {system: true});
         // Store the new fields for reconnect.
-        this._chatRoomFields =
+        this.chatRoomFields =
           this._account.getChatRoomDefaultFieldValues(newFields);
       }
       else if (aNewMode[i] == "b") {
@@ -500,6 +498,13 @@ ircChannel.prototype = {
     for (let [nick, mode] of userModes.entries())
       this.getParticipant(nick).setMode(addNewMode, mode, aSetter);
 
+    // If the topic can now be set (and it couldn't previously) or vice versa,
+    // notify the UI. Note that this status can change by either a channel mode
+    // or a user mode changing.
+    if (this.topicSettable != previousTopicSettable)
+      this.notifyObservers(this, "chat-update-topic");
+
+    // If no channel modes were being set, don't display a message for it.
     if (!channelModes.length)
       return;
 
@@ -511,11 +516,6 @@ ircChannel.prototype = {
             aSetter);
     this.writeMessage(aSetter, msg, {system: true});
 
-    // If the topic can now be set (and it couldn't previously) or vice versa,
-    // notify the UI.
-    if (this.topicSettable != previousTopicSettable)
-      this.notifyObservers(this, "chat-update-topic");
-
     this._receivedInitialMode = true;
   },
 
@@ -810,10 +810,16 @@ ircAccount.prototype = {
   shouldAuthenticate: true,
   // Whether the user has successfully authenticated with NickServ.
   isAuthenticated: false,
+  // The current in use nickname.
+  _nickname: null,
   // The nickname stored in the account name.
   _accountNickname: null,
-  // The nickname that will be used when connecting.
+  // The nickname that was last requested by the user.
   _requestedNickname: null,
+  // The nickname that was last requested. This can differ from
+  // _requestedNickname when a new nick is automatically generated (e.g. by
+  // adding digits).
+  _sentNickname: null,
   // The prefix minus the nick (!user@host) as returned by the server, this is
   // necessary for guessing message lengths.
   prefix: null,
@@ -825,10 +831,9 @@ ircAccount.prototype = {
   maxMessageLength: 512, // 2.3 Messages
   maxHostnameLength: 63, // 2.3.1 Message format in Augmented BNF
 
-  // The default prefixes.
-  userPrefixes: ["@", "!", "%", "+"],
   // The default prefixes to modes.
   userPrefixToModeMap: {"@": "o", "!": "n", "%": "h", "+": "v"},
+  get userPrefixes() Object.keys(this.userPrefixToModeMap),
   // Modes that have a nickname parameter and affect a participant. See 4.1
   // Member Status of RFC 2811.
   memberStatuses: ["a", "h", "o", "O", "q", "v", "!"],
@@ -1177,6 +1182,13 @@ ircAccount.prototype = {
     }
   },
 
+  /*
+   * Ask the server to change the user's nick.
+   */
+  changeNick: function(aNewNick) {
+    this._sentNickname = aNewNick;
+    this.sendMessage("NICK", aNewNick); // Nick message.
+  },
   /*
    * Generate a new nick to change to if the user requested nick is already in
    * use or is otherwise invalid.
@@ -1201,12 +1213,13 @@ ircAccount.prototype = {
     if (oldIndex != -1 && oldIndex < allNicks.length - 1) {
       let newNick = allNicks[oldIndex + 1];
       this.LOG(aOldNick + " is already in use, trying " + newNick);
-      this.sendMessage("NICK", newNick); // Nick message.
+      this.changeNick(newNick);
       return true;
     }
 
     // Separate the nick into the text and digits part.
-    let nickParts = /^(.+?)(\d*)$/.exec(aOldNick);
+    let kNickPattern = /^(.+?)(\d*)$/;
+    let nickParts = kNickPattern.exec(aOldNick);
     let newNick = nickParts[1];
 
     // No nick found from the user's preferences, so just generating one.
@@ -1222,12 +1235,23 @@ ircAccount.prototype = {
         newDigits = "0".repeat(numLeadingZeros) + newDigits;
     }
 
-    // If the nick will be too long, ensure all the digits fit.
-    if (newNick.length + newDigits.length > this.maxNicknameLength) {
+    // Servers truncate nicks that are too long, compare the previously sent
+    // nickname with the returned nickname and check for truncation.
+    if (aOldNick.length < this._sentNickname.length) {
+      // The nick will be too long, overwrite the end of the nick instead of
+      // appending.
+      let maxLength = aOldNick.length;
+
+      let sentNickParts = kNickPattern.exec(this._sentNickname);
+      // Resend the same digits as last time, but overwrite part of the nick
+      // this time.
+      if (nickParts[2] && sentNickParts[2])
+        newDigits = sentNickParts[2];
+
       // Handle the silly case of a single letter followed by all nines.
       if (newDigits.length == this.maxNicknameLength)
         newDigits = newDigits.slice(1);
-      newNick = newNick.slice(0, this.maxNicknameLength - newDigits.length);
+      newNick = newNick.slice(0, maxLength - newDigits.length);
     }
     // Append the digits.
     newNick += newDigits;
@@ -1243,7 +1267,7 @@ ircAccount.prototype = {
     }
 
     this.LOG(aOldNick + " is already in use, trying " + newNick);
-    this.sendMessage("NICK", newNick); // Nick message.
+    this.changeNick(newNick);
     return true;
   },
 
@@ -1356,9 +1380,19 @@ ircAccount.prototype = {
     this._isOnTimer = setTimeout(this.sendIsOn.bind(this), this._isOnDelay);
   },
 
+  // The message of the day uses two fields to append messages.
+  _motd: null,
+  _motdTimer: null,
+
   connect: function() {
     this.reportConnecting();
 
+    // Mark existing MUCs as joining if they will be rejoined.
+    this.conversations.forEach(conversation => {
+      if (conversation.isChat && conversation.chatRoomFields)
+        conversation.joining = true;
+    });
+
     // Load preferences.
     this._port = this.getInt("port");
     this._ssl = this.getBool("ssl");
@@ -1428,8 +1462,10 @@ ircAccount.prototype = {
   // aComponents implements prplIChatRoomFieldValues.
   joinChat: function(aComponents) {
     let channel = aComponents.getValue("channel");
+    // Mildly sanitize input.
+    channel = channel.trimLeft().split(",")[0].split(" ")[0];
     if (!channel) {
-      this.ERROR("joinChat called without a channel name.");
+      this.ERROR("joinChat called without a valid channel name.");
       return null;
     }
 
@@ -1450,15 +1486,19 @@ ircAccount.prototype = {
     let key = aComponents.getValue("password");
     if (key)
       params.push(key);
-    let defaultName = key ? channel + " " + key : channel;
+
     // Send the join command, but don't log the channel key.
     this.sendMessage("JOIN", params,
                      "JOIN " + channel + (key ? " <key not logged>" : ""));
+
     // Open conversation early for better responsiveness.
     let conv = this.getConversation(channel);
-    // Store the prplIChatRoomFieldValues to enable later reconnections.
-    conv._chatRoomFields = this.getChatRoomDefaultFieldValues(defaultName);
     conv.joining = true;
+
+    // Store the prplIChatRoomFieldValues to enable later reconnections.
+    let defaultName = key ? channel + " " + key : channel;
+    conv.chatRoomFields = this.getChatRoomDefaultFieldValues(defaultName);
+
     return conv;
   },
 
@@ -1615,7 +1655,7 @@ ircAccount.prototype = {
     }
 
     // Send the nick message (section 3.1.2).
-    this.sendMessage("NICK", this._requestedNickname);
+    this.changeNick(this._requestedNickname);
 
     // Send the user message (section 3.1.3).
     let username;
@@ -1653,6 +1693,11 @@ ircAccount.prototype = {
     clearTimeout(this._isOnTimer);
     delete this._isOnTimer;
 
+    // MOTD will be resent.
+    delete this._motd;
+    clearTimeout(this._motdTimer)
+    delete this._motdTimer;
+
     // We must authenticate if we reconnect.
     delete this.isAuthenticated;
 
