@@ -37,7 +37,6 @@
 
 // gecko
 #include "nsLayoutCID.h"
-#include "nsIMarkupDocumentViewer.h"
 #include "nsIContentViewer.h"
 
 // embedding
@@ -140,7 +139,6 @@ class nsSaveMsgListener : public nsIUrlListener,
 {
 public:
   nsSaveMsgListener(nsIFile *file, nsMessenger *aMessenger, nsIUrlListener *aListener);
-  virtual ~nsSaveMsgListener();
 
   NS_DECL_ISUPPORTS
 
@@ -180,6 +178,9 @@ public:
   bool    mUrlHasStopped;
   bool    mRequestHasStopped;
   nsresult InitializeDownload(nsIRequest * aRequest, uint32_t aBytesDownloaded);
+
+private:
+  virtual ~nsSaveMsgListener();
 };
 
 class nsSaveAllAttachmentsState
@@ -293,12 +294,8 @@ NS_IMETHODIMP nsMessenger::SetDisplayCharset(const nsACString& aCharset)
     mDocShell->GetContentViewer(getter_AddRefs(cv));
     if (cv)
     {
-      nsCOMPtr<nsIMarkupDocumentViewer> muDV = do_QueryInterface(cv);
-      if (muDV)
-      {
-        muDV->SetHintCharacterSet(aCharset);
-        muDV->SetHintCharacterSetSource(kCharsetFromChannel);
-      }
+      cv->SetHintCharacterSet(aCharset);
+      cv->SetHintCharacterSetSource(kCharsetFromChannel);
 
       mCurrentDisplayCharset = aCharset;
     }
@@ -2476,7 +2473,6 @@ public:
 
 public:
   nsDelAttachListener();
-  virtual ~nsDelAttachListener();
   nsresult StartProcessing(nsMessenger * aMessenger, nsIMsgWindow * aMsgWindow,
     nsAttachmentState * aAttach, bool aSaveFirst);
   nsresult DeleteOriginalMessage();
@@ -2507,6 +2503,9 @@ public:
   bool mWrittenExtra;
   bool mDetaching;
   nsTArray<nsCString> mDetachedFileUris;
+
+private:
+  virtual ~nsDelAttachListener();
 };
 
 //
