@@ -11,11 +11,34 @@
 
 load("../../../resources/logHelper.js");
 load("../../../resources/asyncTestUtils.js");
+load("../../../resources/abSetup.js");
 
 load("../../../resources/messageGenerator.js");
 load("../../../resources/messageModifier.js");
 load("../../../resources/messageInjection.js");
 
+Components.utils.import("resource:///modules/jsTreeSelection.js");
+
+// a fake box needed to get nsMsgDBView to operate on selected messages.
+// Warning: this is a partial implementation. If someone adds additional
+// calls to these objects in nsMsgDBView and friends, it will also
+// be necessary to add fake versions of those calls here.
+function FakeBox(treeView) {
+  this.view = treeView;
+}
+FakeBox.prototype = {
+  view: null,
+  invalidate: function() {},
+  invalidateRow: function() {},
+  beginUpdateBatch: function() {},
+  endUpdateBatch: function() {},
+  invalidateRange: function(startIndex, endIndex) {},
+  rowCountChanged: function (index, count) {},
+
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsITreeBoxObject]),
+};
+
+// Items used to add messages to the folder
 var gMessageGenerator = new MessageGenerator();
 var gScenarioFactory = new MessageScenarioFactory(gMessageGenerator);
 
@@ -275,6 +298,9 @@ function setup_view(aViewType, aViewFlags, aTestFolder) {
   gDBView.curCustomColumn = "authorFirstLetterCol";
 
   gTreeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
+  let fakeBox = new FakeBox(gTreeView);
+  gTreeView.setTree(fakeBox);
+  gTreeView.selection = new JSTreeSelection(fakeBox);
 }
 
 /**
@@ -463,7 +489,7 @@ function  test_number_of_messages() {
   // Bug 574799
   if (gDBView.numMsgsInView != gTestFolder.getTotalMessages(false))
     do_throw("numMsgsInView is " + gDBView.numMsgsInView + " but should be " +
-               aTestFolder.getTotalMessages(false) + "\n");
+             gTestFolder.getTotalMessages(false) + "\n");
   // Bug 600140
   // Maybe elided so open it, now only consider the first one
   if (gDBView.isContainer(0) && !gDBView.isContainerOpen(0))
@@ -480,6 +506,89 @@ function  test_number_of_messages() {
              gDBView.numMsgsInView + "\n");
 }
 
+function test_selected_messages() {
+  gDBView.doCommand(Ci.nsMsgViewCommandType.expandAll);
+
+  // Select one message
+  gTreeView.selection.select(1);
+  let selectedMessages = gDBView.getSelectedMsgHdrs();
+
+  if (selectedMessages.length != 1)
+    do_throw("getSelectedMsgHdrs.length is " + selectedMessages.length +
+             " but should be 1\n");
+
+  let firstSelectedMsg = gDBView.hdrForFirstSelectedMessage
+  if (selectedMessages[0] != firstSelectedMsg)
+    do_throw("getSelectedMsgHdrs[0] is " + selectedMessages[0].messageKey +
+             " but should be " + firstSelectedMsg.messageKey +"\n");
+
+  // Select all messages
+  gTreeView.selection.selectAll();
+  if (gDBView.numSelected != gTreeView.rowCount)
+    do_throw("numSelected is " + gDBView.numSelected + " but should be " +
+             gTreeView.rowCount + "\n");
+
+  selectedMessages = gDBView.getSelectedMsgHdrs();
+  if (selectedMessages.length != gTestFolder.getTotalMessages(false))
+    do_throw("getSelectedMsgHdrs.length is " + selectedMessages.length +
+             " but should be " + gTestFolder.getTotalMessages(false) + "\n");
+
+  for (let i = 0; i < selectedMessages.length; i++) {
+    let expectedHdr = gDBView.getMsgHdrAt(i);
+    if (selectedMessages.indexOf(expectedHdr) == -1) {
+      view_throw("Expected " + expectedHdr.messageKey + ":" +
+                 expectedHdr.mime2DecodedSubject.substr(0, 30) +
+                 " to be selected, but it wasn't\n");
+    }
+  }
+
+  gTreeView.selection.clearSelection();
+}
+
+function test_insert_remove_view_rows() {
+  // Test insertion/removal into m_keys.
+  let startCount = gTreeView.rowCount;
+  let index = 0;
+  let rows = 3;
+  let msgKey = rows * 1000;
+  let flags = 0;
+  let level = 0;
+  let folder = null;
+  let xfview = gDBView.viewType == ViewType.eShowSearch ||
+               gDBView.viewType == ViewType.eShowVirtualFolderResults;
+  if (xfview)
+    folder = gDBView.getFolderForViewIndex(index);
+
+  gDBView.insertTreeRows(index, rows, msgKey, flags, level, folder);
+  assert_view_row_count(startCount + rows);
+  let key = gDBView.getKeyAt(rows - 1);
+  if (key != msgKey)
+    view_throw("msgKey is " + key + " but should be " + msgKey + "\n");
+  gDBView.removeTreeRows(index, rows);
+  assert_view_row_count(startCount);
+
+  // These should fail.
+  try {
+    gDBView.insertTreeRows(startCount + 10, rows, msgKey, flags, level, folder);
+    view_throw("expected exception not caught; inserting at illegal index \n");
+  }
+  catch (ex) {}
+  try {
+    gDBView.insertTreeRows(index, rows, msgKey, flags, level, folder);
+    gDBView.removeTreeRows(index, gTreeView.rowCount + 10);
+    view_throw("expected exception not caught; removing illegal rows \n");
+  }
+  catch (ex) {
+    gDBView.removeTreeRows(index, rows);
+  }
+  if (xfview)
+    try {
+      gDBView.insertTreeRows(index, rows, msgKey, flags, level, null);
+      view_throw("expected exception not caught; folder required for xfvf view \n");
+    }
+    catch (ex) {}
+}
+
 function test_msg_added_to_search_view() {
   // if the view is a non-grouped search view, test adding a header to
   // the search results, and verify it gets put at top.
@@ -664,6 +773,64 @@ function test_xfvf_threading() {
   gTestFolder = save_gTestFolder;
 }
 
+/*
+ * Tests the sorting order of collapsed threads, not of messages within
+ * threads. Currently limited to testing the sort-threads-by-date case,
+ * sorting both by thread root and by newest message.
+*/
+function test_thread_sorting() {
+  let save_gTestFolder = gTestFolder;
+  gTestFolder = make_empty_folder();
+  let messages = [];
+  // build a hierarchy like this (the UID order corresponds to the date order)
+  //  1
+  //   4
+  //  2
+  //   5
+  //  3
+  let msg1 = gMessageGenerator.makeMessage({age: {days: 1, hours: 10}});
+  let msg2 = gMessageGenerator.makeMessage({age: {days: 1, hours: 9}});
+  let msg3 = gMessageGenerator.makeMessage({age: {days: 1, hours: 8}});
+  let msg4 = gMessageGenerator.makeMessage({age: {days: 1, hours: 7}, inReplyTo: msg1});
+  let msg5 = gMessageGenerator.makeMessage({age: {days: 1, hours: 6}, inReplyTo: msg2});
+  messages = messages.concat([msg1, msg2, msg3, msg4, msg5]);
+
+  let msgSet = new SyntheticMessageSet(messages);
+
+  add_sets_to_folders(gTestFolder, [msgSet]);
+
+  // test the non-default pref state first, so the pref gets left with its
+  // default value at the end
+  Services.prefs.setBoolPref("mailnews.sort_threads_by_root", true);
+  gDBView.open(gTestFolder, SortType.byDate, SortOrder.ascending, ViewFlags.kThreadedDisplay, {});
+
+  assert_view_row_count(3);
+  assert_view_message_at_indices(msg1, 0);
+  assert_view_message_at_indices(msg2, 1);
+  assert_view_message_at_indices(msg3, 2);
+
+  gDBView.sort(SortType.byDate, SortOrder.descending);
+  assert_view_message_at_indices(msg3, 0);
+  assert_view_message_at_indices(msg2, 1);
+  assert_view_message_at_indices(msg1, 2);
+
+  Services.prefs.clearUserPref("mailnews.sort_threads_by_root");
+  gDBView.open(gTestFolder, SortType.byDate, SortOrder.ascending, ViewFlags.kThreadedDisplay, {});
+
+  assert_view_row_count(3);
+  assert_view_message_at_indices(msg3, 0);
+  assert_view_message_at_indices(msg1, 1);
+  assert_view_message_at_indices(msg2, 2);
+
+  gDBView.sort(SortType.byDate, SortOrder.descending);
+  assert_view_message_at_indices(msg2, 0);
+  assert_view_message_at_indices(msg1, 1);
+  assert_view_message_at_indices(msg3, 2);
+
+  gDBView.close();
+  gTestFolder = save_gTestFolder;
+}
+
 var view_types = [
   ["threaded", ViewFlags.kThreadedDisplay],
   ["quicksearch", ViewFlags.kThreadedDisplay],
@@ -676,7 +843,9 @@ var view_types = [
 
 var tests_for_all_views = [
   test_sort_columns,
-  test_number_of_messages
+  test_number_of_messages,
+  test_selected_messages,
+  test_insert_remove_view_rows
 ];
 
 var tests_for_specific_views = {
@@ -684,7 +853,8 @@ var tests_for_specific_views = {
     test_group_dummies_under_mutation_by_date
   ],
   threaded: [
-    test_expand_collapse
+    test_expand_collapse,
+    test_thread_sorting
   ],
   search: [
     test_msg_added_to_search_view
