@@ -73,7 +73,7 @@ nsSmtpService::~nsSmtpService()
 
 }
 
-NS_IMPL_ISUPPORTS2(nsSmtpService, nsISmtpService, nsIProtocolHandler)
+NS_IMPL_ISUPPORTS(nsSmtpService, nsISmtpService, nsIProtocolHandler)
 
 
 NS_IMETHODIMP nsSmtpService::SendMailMessage(nsIFile * aFilePath,
@@ -325,7 +325,7 @@ NS_IMETHODIMP nsSmtpService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
   nsCOMPtr<nsIAsyncInputStream> pipeIn;
   nsCOMPtr<nsIAsyncOutputStream> pipeOut;
   nsCOMPtr<nsIPipe> pipe = do_CreateInstance("@mozilla.org/pipe;1");
-  nsresult rv = pipe->Init(false, false, 0, 0, nullptr);
+  nsresult rv = pipe->Init(false, false, 0, 0);
   if (NS_FAILED(rv)) 
     return rv;
   
@@ -695,9 +695,9 @@ nsSmtpService::findServerByHostname(nsISmtpServer *aServer, void *aData)
   bool checkUsername = !entry->username.IsEmpty();
     
   if ((!checkHostname ||
-       (entry->hostname.Equals(hostname, nsCaseInsensitiveCStringComparator())) &&
+       (entry->hostname.Equals(hostname, nsCaseInsensitiveCStringComparator()))) &&
        (!checkUsername ||
-        entry->username.Equals(username, nsCaseInsensitiveCStringComparator()))))
+        entry->username.Equals(username, nsCaseInsensitiveCStringComparator())))
   {
     entry->server = aServer;
     return false;        // stop when found
