@@ -14,7 +14,7 @@ FeedParser.prototype =
   // parseFeed() returns an array of parsed items ready for processing.  It is
   // currently a synchronous operation.  If there is an error parsing the feed,
   // parseFeed returns an empty feed in addition to calling aFeed.onParseError.
-  parseFeed: function (aFeed, aDOM, aBaseURI)
+  parseFeed: function (aFeed, aDOM)
   {
     if (!(aDOM instanceof Ci.nsIDOMXMLDocument))
     {
@@ -45,7 +45,7 @@ FeedParser.prototype =
       // the XMLHttpRequest.responseBody property that IE has, which provides
       // access to the unencoded response.
       let xmlString = this.mSerializer.serializeToString(doc);
-      return this.parseAsRSS1(aFeed, xmlString, aBaseURI);
+      return this.parseAsRSS1(aFeed, xmlString, aFeed.request.channel.URI);
     }
     else if (doc.namespaceURI == FeedUtils.ATOM_03_NS)
     {
@@ -128,6 +128,7 @@ FeedParser.prototype =
       let item = new FeedItem();
       item.feed = aFeed;
       item.enclosures = [];
+      item.keywords = [];
 
       tags = this.childrenByTagNameNS(itemNode, FeedUtils.FEEDBURNER_NS, "origLink");
       let link = this.getNodeValue(tags ? tags[0] : null);
@@ -270,6 +271,19 @@ FeedParser.prototype =
           item.enclosures.push(new FeedEnclosure(origEncUrl));
       }
 
+      // Support <category> and autotagging.
+      tags = this.childrenByTagNameNS(itemNode, nsURI, "category");
+      if (tags)
+      {
+        for (let tag of tags)
+        {
+          let term = this.getNodeValue(tag);
+          term = term ? this.xmlUnescape(term.replace(",", ";")) : null;
+          if (term && item.keywords.indexOf(term) == -1)
+            item.keywords.push(term);
+        }
+      }
+
       parsedItems.push(item);
     }
 
@@ -326,7 +340,7 @@ FeedParser.prototype =
 
       // Prefer the value of the link tag to the item URI since the URI could be
       // a relative URN.
-      let uri = itemResource.Value;
+      let uri = itemResource.ValueUTF8;
       let link = this.getRDFTargetValue(ds, itemResource, FeedUtils.RSS_LINK);
       item.url = link || uri;
       item.description = this.getRDFTargetValue(ds, itemResource,
@@ -534,6 +548,7 @@ FeedParser.prototype =
       let item = new FeedItem();
       item.feed = aFeed;
       item.enclosures = [];
+      item.keywords = [];
 
       tags = this.childrenByTagNameNS(itemNode, FeedUtils.FEEDBURNER_NS, "origLink");
       item.url = this.getNodeValue(tags ? tags[0] : null);
@@ -553,7 +568,7 @@ FeedParser.prototype =
       if (!item.title || !item.id)
       {
         // We're lenient about other mandatory tags, but insist on these.
-        FeedUtils.log.info("FeedParser.parseAsAtom: <entry> missing mandatory " +
+        FeedUtils.log.info("FeedParser.parseAsAtomIETF: <entry> missing mandatory " +
                            "element <id>, or <title> and no <summary>; skipping");
         continue;
       }
@@ -612,7 +627,7 @@ FeedParser.prototype =
         for (let tag of tags)
         {
           let url = tag.getAttribute("rel") == "enclosure" ?
-                      (tag.getAttribute("url") || "").trim() : null;
+                      (tag.getAttribute("href") || "").trim() : null;
           if (url && encUrls.indexOf(url) == -1)
           {
             item.enclosures.push(new FeedEnclosure(url,
@@ -648,6 +663,19 @@ FeedParser.prototype =
         item.inReplyTo = item.inReplyTo.trimRight();
       }
 
+      // Support <category> and autotagging.
+      tags = this.childrenByTagNameNS(itemNode, FeedUtils.ATOM_IETF_NS, "category");
+      if (tags)
+      {
+        for (let tag of tags)
+        {
+          let term = tag.getAttribute("term");
+          term = term ? this.xmlUnescape(term.replace(",", ";")).trim() : null;
+          if (term && item.keywords.indexOf(term) == -1)
+            item.keywords.push(term);
+        }
+      }
+
       parsedItems.push(item);
     }
 
@@ -726,7 +754,7 @@ FeedParser.prototype =
       }
 
       if (ret)
-        return ret;
+        return ret.trim();
     }
 
     return null;
