@@ -48,7 +48,7 @@
 #include "nsMsgBaseCID.h"
 
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
 #include "nsEudoraWin32.h"
 #endif
 #ifdef XP_MACOSX
@@ -64,7 +64,6 @@ class ImportEudoraMailImpl : public nsIImportMail
 {
 public:
   ImportEudoraMailImpl();
-  virtual ~ImportEudoraMailImpl();
 
   static nsresult Create(nsIImportMail** aImport);
 
@@ -96,10 +95,11 @@ public:
 
 
 private:
+  virtual ~ImportEudoraMailImpl();
   static void  ReportSuccess(nsString& name, int32_t count, nsString *pStream);
 
 private:
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
   nsEudoraWin32  m_eudora;
 #endif
 #ifdef XP_MACOSX
@@ -113,7 +113,6 @@ class ImportEudoraAddressImpl : public nsIImportAddressBooks
 {
 public:
   ImportEudoraAddressImpl();
-  virtual ~ImportEudoraAddressImpl();
 
   static nsresult Create(nsIImportAddressBooks** aImport);
 
@@ -151,10 +150,11 @@ public:
   NS_IMETHOD SetSampleLocation(nsIFile *) { return NS_OK; }
 
 private:
+  virtual ~ImportEudoraAddressImpl();
   static void  ReportSuccess(nsString& name, nsString *pStream);
 
 private:
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
   nsEudoraWin32  m_eudora;
 #endif
 #ifdef XP_MACOSX
@@ -185,7 +185,7 @@ nsEudoraImport::~nsEudoraImport()
 
 
 
-NS_IMPL_ISUPPORTS1(nsEudoraImport, nsIImportModule)
+NS_IMPL_ISUPPORTS(nsEudoraImport, nsIImportModule)
 
 
 NS_IMETHODIMP nsEudoraImport::GetName(char16_t **name)
@@ -363,7 +363,7 @@ ImportEudoraMailImpl::ImportEudoraMailImpl()
     // Use one dummy entry for now as a placeholder to keep the Mac code valid,
     // until we enter actual reasonable defaults for Mac builds.
     EudoraDefaultLabels    defaultEudoraLabels[1] =
-                    { "eudoralabel1", NS_LITERAL_STRING("Label 1"), "#FF6600" };
+                    { { "eudoralabel1", NS_LITERAL_STRING("Label 1"), "#FF6600" } };
 #else
     // These aren't the actual default Windows Eudora colors. Rather they're the closest
     // equivalents that I could find that Thunderbird supports. When importing actual
@@ -406,7 +406,7 @@ ImportEudoraMailImpl::~ImportEudoraMailImpl()
 
 
 
-NS_IMPL_ISUPPORTS1(ImportEudoraMailImpl, nsIImportMail)
+NS_IMPL_ISUPPORTS(ImportEudoraMailImpl, nsIImportMail)
 
 NS_IMETHODIMP ImportEudoraMailImpl::GetDefaultLocation(nsIFile **ppLoc, bool *found, bool *userVerify)
 {
@@ -605,7 +605,7 @@ ImportEudoraAddressImpl::~ImportEudoraAddressImpl()
 
 
 
-NS_IMPL_ISUPPORTS1(ImportEudoraAddressImpl, nsIImportAddressBooks)
+NS_IMPL_ISUPPORTS(ImportEudoraAddressImpl, nsIImportAddressBooks)
 
 
 NS_IMETHODIMP ImportEudoraAddressImpl::GetAutoFind(char16_t **description, bool *_retval)
