@@ -89,7 +89,7 @@ nsNNTPNewsgroupList::~nsNNTPNewsgroupList()
   CleanUp();
 }
 
-NS_IMPL_ISUPPORTS2(nsNNTPNewsgroupList, nsINNTPNewsgroupList, nsIMsgFilterHitNotify)
+NS_IMPL_ISUPPORTS(nsNNTPNewsgroupList, nsINNTPNewsgroupList, nsIMsgFilterHitNotify)
 
 nsresult
 nsNNTPNewsgroupList::Initialize(nsINntpUrl *runningURL, nsIMsgNewsFolder *newsFolder)
@@ -125,7 +125,7 @@ nsNNTPNewsgroupList::Initialize(nsINntpUrl *runningURL, nsIMsgNewsFolder *newsFo
   // servArray may have duplicates already in m_filterHeaders.
   for (uint32_t i = 0; i < servArray.Length(); i++)
   {
-    if (m_filterHeaders.IndexOf(servArray[i]) == m_filterHeaders.NoIndex)
+    if (!m_filterHeaders.Contains(servArray[i]))
       m_filterHeaders.AppendElement(servArray[i]);
   }
   return NS_OK;
@@ -312,7 +312,7 @@ nsNNTPNewsgroupList::GetRangeOfArtsToDownload(nsIMsgWindow *aMsgWindow,
     rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = bundle->GetStringFromName(NS_LITERAL_STRING("noNewMessages").get(), getter_Copies(statusString));
+    rv = bundle->GetStringFromName(MOZ_UTF16("noNewMessages"), getter_Copies(statusString));
     NS_ENSURE_SUCCESS(rv, rv);
 
     SetProgressStatus(statusString.get());
@@ -624,14 +624,13 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
   // you can't move news messages, so applyMore is always true
   *aApplyMore = true;
 
-  nsCOMPtr<nsISupportsArray> filterActionList;
-  nsresult rv = NS_NewISupportsArray(getter_AddRefs(filterActionList));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = aFilter->GetSortedActionList(filterActionList);
+  nsCOMPtr<nsIArray> filterActionList;
+
+  nsresult rv = aFilter->GetSortedActionList(getter_AddRefs(filterActionList));
   NS_ENSURE_SUCCESS(rv, rv);
 
   uint32_t numActions;
-  rv = filterActionList->Count(&numActions);
+  rv = filterActionList->GetLength(&numActions);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool loggingEnabled = false;
@@ -643,8 +642,9 @@ NS_IMETHODIMP nsNNTPNewsgroupList::ApplyFilterHit(nsIMsgFilter *aFilter, nsIMsgW
   for (uint32_t actionIndex = 0; actionIndex < numActions; actionIndex++)
   {
     nsCOMPtr<nsIMsgRuleAction> filterAction;
-    filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction), getter_AddRefs(filterAction));
-    if (!filterAction)
+    rv = filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction),
+                                                       getter_AddRefs(filterAction));
+    if (NS_FAILED(rv) || !filterAction)
       continue;
 
     nsMsgRuleActionType actionType;
@@ -886,8 +886,8 @@ nsNNTPNewsgroupList::FinishXOVERLINE(int status, int *newstatus)
       rv = bundleService->CreateBundle(NEWS_MSGS_URL, getter_AddRefs(bundle));
       NS_ENSURE_SUCCESS(rv, rv);
 
-      const PRUnichar *formatStrings[2] = { firstStr.get(), lastStr.get() };
-      rv = bundle->FormatStringFromName(NS_LITERAL_STRING("downloadingArticles").get(), formatStrings, 2, getter_Copies(statusString));
+      const char16_t *formatStrings[2] = { firstStr.get(), lastStr.get() };
+      rv = bundle->FormatStringFromName(MOZ_UTF16("downloadingArticles"), formatStrings, 2, getter_Copies(statusString));
       NS_ENSURE_SUCCESS(rv, rv);
 
       SetProgressStatus(statusString.get());
@@ -930,15 +930,14 @@ nsNNTPNewsgroupList::ProcessXHDRLine(const nsACString &line)
   if (key.CharAt(0) < '0' || key.CharAt(0) > '9')
     return NS_OK;
 
-  nsresult code;
-  int32_t number = key.ToInteger(&code);
-  if (code != NS_OK)
-    return NS_ERROR_FAILURE;
+  nsresult rv;
+  int32_t number = key.ToInteger(&rv);
+  NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
   // RFC 2980 specifies one or more spaces.
   value.Trim(" ");
 
-  nsCOMPtr <nsIMsgDBHdr> header;
-  nsresult rv = m_newsDB->GetMsgHdrForKey(number, getter_AddRefs(header));
+  nsCOMPtr<nsIMsgDBHdr> header;
+  rv = m_newsDB->GetMsgHdrForKey(number, getter_AddRefs(header));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = header->SetStringProperty(m_filterHeaders[m_currentXHDRIndex].get(), value.get());
@@ -1087,7 +1086,7 @@ nsNNTPNewsgroupList::AddHeader(const char *header, const char *value)
   {
     rv = m_newMsgHdr->SetLineCount(atol(value));
   }
-  else if (m_filterHeaders.IndexOf(nsDependentCString(header)) != m_filterHeaders.NoIndex)
+  else if (m_filterHeaders.Contains(nsDependentCString(header)))
   {
     rv = m_newMsgHdr->SetStringProperty(header, value);
   }
@@ -1229,7 +1228,7 @@ nsNNTPNewsgroupList::SetProgressBarPercent(int32_t percent)
 }
 
 void
-nsNNTPNewsgroupList::SetProgressStatus(const PRUnichar *message)
+nsNNTPNewsgroupList::SetProgressStatus(const char16_t *aMessage)
 {
   if (!m_runningURL)
     return;
@@ -1240,7 +1239,25 @@ nsNNTPNewsgroupList::SetProgressStatus(const PRUnichar *message)
     mailnewsUrl->GetStatusFeedback(getter_AddRefs(feedback));
 
     if (feedback) {
-      feedback->ShowStatusString(nsDependentString(message));
+      // prepending the account name to the status message.
+      nsresult rv;
+      nsCOMPtr <nsIMsgIncomingServer> server;
+      rv = mailnewsUrl->GetServer(getter_AddRefs(server));
+      NS_ENSURE_SUCCESS_VOID(rv);
+      nsString accountName;
+      server->GetPrettyName(accountName);
+      nsString statusMessage;
+      nsCOMPtr<nsIStringBundleService> sbs =
+        mozilla::services::GetStringBundleService();
+      nsCOMPtr<nsIStringBundle> bundle;
+      rv = sbs->CreateBundle(MSGS_URL,
+                             getter_AddRefs(bundle));
+      NS_ENSURE_SUCCESS_VOID(rv);
+      const char16_t *params[] = { accountName.get(), aMessage };
+      bundle->FormatStringFromName(MOZ_UTF16("statusMessage"),
+                                   params, 2, getter_Copies(statusMessage));
+
+      feedback->ShowStatusString(statusMessage);
     }
   }
 }
@@ -1277,16 +1294,16 @@ nsNNTPNewsgroupList::UpdateStatus(bool filtering, int32_t numDLed, int32_t totTo
   if (filtering)
   {
     NS_ConvertUTF8toUTF16 header(m_filterHeaders[m_currentXHDRIndex]);
-    const PRUnichar *formatStrings[4] = { header.get(),
+    const char16_t *formatStrings[4] = { header.get(),
       numDownloadedStr.get(), totalToDownloadStr.get(), newsgroupName.get() };
-    rv = bundle->FormatStringFromName(NS_LITERAL_STRING("newNewsgroupFilteringHeaders").get(),
+    rv = bundle->FormatStringFromName(MOZ_UTF16("newNewsgroupFilteringHeaders"),
       formatStrings, 4, getter_Copies(statusString));
   }
   else
   {
-    const PRUnichar *formatStrings[3] = { numDownloadedStr.get(),
+    const char16_t *formatStrings[3] = { numDownloadedStr.get(),
       totalToDownloadStr.get(), newsgroupName.get() };
-    rv = bundle->FormatStringFromName(NS_LITERAL_STRING("newNewsgroupHeaders").get(),
+    rv = bundle->FormatStringFromName(MOZ_UTF16("newNewsgroupHeaders"),
       formatStrings, 3, getter_Copies(statusString));
   }
   if (!NS_SUCCEEDED(rv))
