@@ -10,9 +10,14 @@
 #include "DOMMediaStream.h"
 #include "MediaStreamGraph.h"
 #include "mozilla/dom/MediaStreamTrackBinding.h"
+#include "mozilla/dom/VideoStreamTrack.h"
 
 namespace mozilla {
 
+namespace dom {
+class File;
+}
+
 struct VideoTrackConstraintsN;
 struct AudioTrackConstraintsN;
 
@@ -46,7 +51,7 @@ enum MediaSourceType {
   Screen = (int) dom::MediaSourceEnum::Screen,
   Application = (int) dom::MediaSourceEnum::Application,
   Window, // = (int) dom::MediaSourceEnum::Window, // XXX bug 1038926
-  //Browser = (int) dom::MediaSourceEnum::Browser, // proposed in WG, unclear if it's useful
+  Browser = (int) dom::MediaSourceEnum::Browser, // proposed in WG, unclear if it's useful
   Microphone
 };
 
@@ -107,12 +112,6 @@ public:
   /* tell the source if there are any direct listeners attached */
   virtual void SetDirectListeners(bool) = 0;
 
-  /* Take a snapshot from this source. In the case of video this is a single
-   * image, and for audio, it is a snippet lasting aDuration milliseconds. The
-   * duration argument is ignored for a MediaEngineVideoSource.
-   */
-  virtual nsresult Snapshot(uint32_t aDuration, nsIDOMFile** aFile) = 0;
-
   /* Called when the stream wants more data */
   virtual void NotifyPull(MediaStreamGraph* aGraph,
                           SourceMediaStream *aSource,
@@ -137,6 +136,29 @@ public:
   /* Returns the type of media source (camera, microphone, screen, window, etc) */
   virtual const MediaSourceType GetMediaSource() = 0;
 
+  // Callback interface for TakePhoto(). Either PhotoComplete() or PhotoError()
+  // should be called.
+  class PhotoCallback {
+  public:
+    NS_INLINE_DECL_THREADSAFE_REFCOUNTING(PhotoCallback)
+
+    // aBlob is the image captured by MediaEngineSource. It is
+    // called on main thread.
+    virtual nsresult PhotoComplete(already_AddRefed<dom::File> aBlob) = 0;
+
+    // It is called on main thread. aRv is the error code.
+    virtual nsresult PhotoError(nsresult aRv) = 0;
+
+  protected:
+    virtual ~PhotoCallback() {}
+  };
+
+  /* If implementation of MediaEngineSource supports TakePhoto(), the picture
+   * should be return via aCallback object. Otherwise, it returns NS_ERROR_NOT_IMPLEMENTED.
+   * Currently, only Gonk MediaEngineSource implementation supports it.
+   */
+  virtual nsresult TakePhoto(PhotoCallback* aCallback) = 0;
+
   /* Return false if device is currently allocated or started */
   bool IsAvailable() {
     if (mState == kAllocated || mState == kStarted) {
@@ -150,6 +172,8 @@ public:
    * a Start(). Only Allocate() may be called after a Deallocate(). */
 
 protected:
+  // Only class' own members can be initialized in constructor initializer list.
+  explicit MediaEngineSource(MediaEngineState aState) : mState(aState) {}
   MediaEngineState mState;
 };
 
@@ -203,12 +227,14 @@ class MediaEngineVideoSource : public MediaEngineSource
 public:
   virtual ~MediaEngineVideoSource() {}
 
-  virtual const MediaSourceType GetMediaSource() {
-      return MediaSourceType::Camera;
-  }
   /* This call reserves but does not start the device. */
   virtual nsresult Allocate(const VideoTrackConstraintsN &aConstraints,
                             const MediaEnginePrefs &aPrefs) = 0;
+protected:
+  explicit MediaEngineVideoSource(MediaEngineState aState)
+    : MediaEngineSource(aState) {}
+  MediaEngineVideoSource()
+    : MediaEngineSource(kReleased) {}
 };
 
 /**
@@ -222,6 +248,11 @@ public:
   /* This call reserves but does not start the device. */
   virtual nsresult Allocate(const AudioTrackConstraintsN &aConstraints,
                             const MediaEnginePrefs &aPrefs) = 0;
+protected:
+  explicit MediaEngineAudioSource(MediaEngineState aState)
+    : MediaEngineSource(aState) {}
+  MediaEngineAudioSource()
+    : MediaEngineSource(kReleased) {}
 
 };
 
