@@ -97,6 +97,17 @@ function initPrefs()
     var prefs =
         [
          ["activityFlashDelay", 200,      "hidden"],
+         ["alert.overlapDelay", 50,       "hidden"],
+         ["alert.floodDensity", 290,      "hidden"],
+         ["alert.floodDispersion", 200,   "hidden"],
+         ["alert.enabled",      true,     ".palert"],
+         ["alert.globalEnabled", true,    "global.palertconfig"],
+         ["alert.clickable",    true,     "hidden"],
+         ["alert.nonFocusedOnly", true,   "global.palertconfig"],
+         ["alert.channel.event", false,   ".palert"],
+         ["alert.channel.chat", false,    ".palert"],
+         ["alert.channel.stalk", true,    ".palert"],
+         ["alert.user.chat",    true,     ".palert"],
          ["aliases",            [],       "lists.aliases"],
          ["autoAwayCap",        300,      "global"],
          ["autoAwayPeriod",     2,        "appearance.misc"],
@@ -109,6 +120,7 @@ function initPrefs()
          ["awayIdleMsg",        "",       ".ident"],
          ["awayIdleTime",       0,        ".ident"],
          ["awayNick",           "",       ".ident"],
+         ["bugKeyword",         "bug",    "appearance.misc"],
          ["bugURL",           "https://bugzilla.mozilla.org/show_bug.cgi?id=%s",
                                           "appearance.misc"],
          ["bugURL.comment",     "#c%s",   "appearance.misc"],
@@ -216,14 +228,14 @@ function initPrefs()
          // Event == "Superfluous" activity.
          // Stalk == "Attention" activity.
          // Start == When view it opened.
-         ["sound.channel.chat",  "",          "global.soundEvts"],
-         ["sound.channel.event", "",          "global.soundEvts"],
-         ["sound.channel.stalk", "beep",      "global.soundEvts"],
-         ["sound.channel.start", "",          "global.soundEvts"],
+         ["sound.channel.chat",  "",          ".soundEvts"],
+         ["sound.channel.event", "",          ".soundEvts"],
+         ["sound.channel.stalk", "beep",      ".soundEvts"],
+         ["sound.channel.start", "",          ".soundEvts"],
          ["sound.enabled",       true,        "global.sounds"],
          ["sound.overlapDelay",  2000,        "global.sounds"],
-         ["sound.user.stalk",    "beep",      "global.soundEvts"],
-         ["sound.user.start",    "beep beep", "global.soundEvts"],
+         ["sound.user.stalk",    "beep",      ".soundEvts"],
+         ["sound.user.start",    "beep beep", ".soundEvts"],
          ["stalkWholeWords",    true,     "lists.stalkWords"],
          ["stalkWords",         [],       "lists.stalkWords"],
          ["tabLabel",           "",       "hidden"],
@@ -400,6 +412,11 @@ function getNetworkPrefManager(network)
 
     var prefs =
         [
+         ["alert.enabled",      defer,     ".palert"],
+         ["alert.channel.event",defer,     ".palert"],
+         ["alert.channel.chat", defer,     ".palert"],
+         ["alert.channel.stalk",defer,     ".palert"],
+         ["alert.user.chat",    defer,     ".palert"],
          ["autoAwayPeriod",   defer, "appearance.misc"],
          ["autoperform",      [],    "lists.autoperform"],
          ["autoRejoin",       defer, ".connect"],
@@ -432,6 +449,12 @@ function getNetworkPrefManager(network)
          ["outputWindowURL",  defer, "hidden"],
          ["proxy.typeOverride", defer, ".connect"],
          ["reconnect",        defer, ".connect"],
+         ["sound.channel.chat",  defer, ".soundEvts"],
+         ["sound.channel.event", defer, ".soundEvts"],
+         ["sound.channel.stalk", defer, ".soundEvts"],
+         ["sound.channel.start", defer, ".soundEvts"],
+         ["sound.user.stalk",    defer, ".soundEvts"],
+         ["sound.user.start",    defer, ".soundEvts"],
          ["tabLabel",         "",     "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
@@ -502,6 +525,10 @@ function getChannelPrefManager(channel)
 
     var prefs =
         [
+         ["alert.enabled",      defer,     ".palert"],
+         ["alert.channel.event",defer,     ".palert"],
+         ["alert.channel.chat", defer,     ".palert"],
+         ["alert.channel.stalk",defer,     ".palert"],
          ["autoperform",      [],    "lists.autoperform"],
          ["autoRejoin",       defer, ".connect"],
          ["bugURL",           defer, "appearance.misc"],
@@ -519,6 +546,10 @@ function getChannelPrefManager(channel)
          ["logFileName",      makeLogNameChannel, "hidden"],
          ["motif.current",    defer, "appearance.motif"],
          ["outputWindowURL",  defer, "hidden"],
+         ["sound.channel.chat",  defer, ".soundEvts"],
+         ["sound.channel.event", defer, ".soundEvts"],
+         ["sound.channel.stalk", defer, ".soundEvts"],
+         ["sound.channel.start", defer, ".soundEvts"],
          ["tabLabel",         "",    "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
@@ -563,6 +594,8 @@ function getUserPrefManager(user)
 
     var prefs =
         [
+         ["alert.enabled",      defer,     ".palert"],
+         ["alert.user.chat",    defer,     ".palert"],
          ["autoperform",      [],    "lists.autoperform"],
          ["charset",          defer, ".connect"],
          ["collapseActions",  defer, "appearance.misc"],
@@ -575,6 +608,8 @@ function getUserPrefManager(user)
          ["logFileName",      makeLogNameUser, "hidden"],
          ["motif.current",    defer, "appearance.motif"],
          ["outputWindowURL",  defer, "hidden"],
+         ["sound.user.stalk",    defer, ".soundEvts"],
+         ["sound.user.start",    defer, ".soundEvts"],
          ["tabLabel",         "",    "hidden"],
          ["timestamps",         defer, "appearance.timestamps"],
          ["timestamps.display", defer, "appearance.timestamps"],
@@ -612,6 +647,8 @@ function getDCCUserPrefManager(user)
 
     var prefs =
         [
+         ["alert.enabled",      defer,     ".palert"],
+         ["alert.user.chat",    defer,     ".palert"],
          ["charset",          defer, ".connect"],
          ["collapseMsgs",     defer, "appearance.misc"],
          ["displayHeader",    client.prefs["dccUserHeader"], "appearance.misc"],
@@ -662,6 +699,11 @@ function onPrefChanged(prefName, newValue, oldValue)
             initIdleAutoAway(newValue);
             break;
 
+        case "bugKeyword":
+            client.munger.delRule("bugzilla-link");
+            addBugzillaLinkMungerRule(newValue, 10, 10);
+            break;
+
         case "channelMaxLines":
             CIRCChannel.prototype.MAX_MESSAGES = newValue;
             break;
@@ -791,6 +833,20 @@ function onPrefChanged(prefName, newValue, oldValue)
             client.dispatch("sync-log");
             break;
 
+        case "alert.globalEnabled":
+            updateAlertIcon();
+            break;
+
+        case "alert.floodDensity":
+            if (client.alert && client.alert.floodProtector)
+                client.alert.floodProtector.floodDensity = newValue;
+            break;
+
+        case "alert.floodDispersion":
+            if (client.alert && client.alert.floodProtector)
+                client.alert.floodProtector.floodDispersion = newValue;
+            break;
+
         case "aliases":
             updateAliases();
             break;
